@@ -79,6 +79,7 @@ from .exceptions import UserError
 from .ipfs_utils import ipfs_available, IPFSException
 from .logging_capture import setup_logging
 from .logging_worker import write_logs_worker
+from .login import do_docker_login
 from .misc import list_all_files
 from .shell_config import DTShellConstants, read_shell_config
 from .types import ContainerName, ServiceName
@@ -435,6 +436,10 @@ def upload_process(
 
         # noinspection PyBroadException
         try:
+            # TODO:
+            more_features = {}
+            features = get_features(more_features, use_ipfs=False)
+
             res_ = dtserver_job_heartbeat(
                 token,
                 job_id=job_id,
@@ -443,6 +448,7 @@ def upload_process(
                 evaluator_version=evaluator_version,
                 impersonate=impersonate,
                 uploaded=_uploaded,
+                features=features,
             )
         except:
             logger.warning(traceback.format_exc())
@@ -454,39 +460,40 @@ def upload_process(
     logger.info("upload_process terminated gracefully")
 
 
-class TermHandler:
-    def __init__(self, *, token, job_id, machine_id, process_id, evaluator_version, impersonate, timeout):
-        self.token = token
-        self.job_id = job_id
-        self.machine_id = machine_id
-        self.process_id = process_id
-        self.evaluator_version = evaluator_version
-        self.impersonate = impersonate
-        self.timeout = timeout
-
-    def sigterm_handler(self, _signo, _stack_frame):
-        s = traceback.format_stack(_stack_frame)
-        msg_ = f"Evaluator received SIGTERM \n\n{s} "
-
-        status = ChallengesConstants.STATUS_JOB_ABORTED
-        cr0 = ChallengeResults(status, msg_, scores={})
-        logger.error(cr0=cr0)
-        dtserver_report_job(
-            token=self.token,
-            job_id=self.job_id,
-            stats=cr0.stats,
-            result=cr0.get_status(),
-            ipfs_hashes=cr0.ipfs_hashes,
-            machine_id=self.machine_id,
-            process_id=self.process_id,
-            evaluator_version=self.evaluator_version,
-            uploaded=[],
-            impersonate=self.impersonate,
-            timeout=self.timeout,
-        )
-
-        # Raises SystemExit(0):
-        sys.exit(23)
+#
+# class TermHandler:
+#     def __init__(self, *, token, job_id, machine_id, process_id, evaluator_version, impersonate, timeout):
+#         self.token = token
+#         self.job_id = job_id
+#         self.machine_id = machine_id
+#         self.process_id = process_id
+#         self.evaluator_version = evaluator_version
+#         self.impersonate = impersonate
+#         self.timeout = timeout
+#
+#     def sigterm_handler(self, _signo, _stack_frame):
+#         s = traceback.format_stack(_stack_frame)
+#         msg_ = f"Evaluator received SIGTERM \n\n{s} "
+#
+#         status = ChallengesConstants.STATUS_JOB_ABORTED
+#         cr0 = ChallengeResults(status, msg_, scores={})
+#         logger.error(cr0=cr0)
+#         dtserver_report_job(
+#             token=self.token,
+#             job_id=self.job_id,
+#             stats=cr0.stats,
+#             result=cr0.get_status(),
+#             ipfs_hashes=cr0.ipfs_hashes,
+#             machine_id=self.machine_id,
+#             process_id=self.process_id,
+#             evaluator_version=self.evaluator_version,
+#             uploaded=[],
+#             impersonate=self.impersonate,
+#             timeout=self.timeout,
+#         )
+#
+#         # Raises SystemExit(0):
+#         sys.exit(23)
 
 
 def go_(
@@ -558,6 +565,10 @@ def go_(
         raise NothingLeft(msg, res=res)
     job_id = res["job_id"]
     aws_config: Optional[AWSConfig] = res.get("aws_config", None)
+
+    sc = read_shell_config()
+    if sc.docker_password and sc.docker_username:
+        do_docker_login(sc.docker_username, sc.docker_password)
     #
     # handler = TermHandler(
     #     token=token,
@@ -600,6 +611,7 @@ def go_(
                 evaluator_version=evaluator_version,
                 impersonate=impersonate,
                 uploaded=[],
+                features=features,
             )
 
             abort = res_["abort"]
