@@ -7,19 +7,37 @@ import shutil
 import subprocess
 import sys
 from datetime import datetime
+from typing import cast
 
 import termcolor
 import yaml
-from duckietown_challenges import CHALLENGE_PREVIOUS_STEPS_DIR, ChallengeResults, List, traceback
-from duckietown_challenges.rest_methods import dtserver_get_compatible_challenges, get_challenge_description
-from duckietown_challenges.submission_read import read_submission_info
-from duckietown_challenges.utils import indent
+from docker import DockerClient
+from zuper_commons.fs import mkdirs_thread_safe, read_ustring_from_utf8_file, write_ustring_to_utf8_file
+from zuper_commons.text import indent
+from zuper_commons.timing import now_utc
 
-from .env_checks import check_docker_environment
+from duckietown_build_utils import (
+    BuildFailed,
+    DockerCompleteImageName,
+    get_duckietown_labels,
+    get_important_env_build_args_dict,
+    run_build,
+    update_versions,
+)
+from duckietown_challenges import (
+    CHALLENGE_PREVIOUS_STEPS_DIR,
+    ChallengeResults,
+    dtserver_get_compatible_challenges,
+    get_challenge_description,
+    List,
+    read_submission_info,
+    SubmissionID,
+    traceback,
+)
+from duckietown_challenges.utils import tag_from_date
+from . import logger
 from .exceptions import UserError
-from .logging import logger
 from .runner import get_token_from_shell_config, run_single
-from .submission_build import build_image
 
 usage = """
 
@@ -27,7 +45,6 @@ usage = """
 
 
 """
-# logger = ZLogger("runner-local")
 
 
 # noinspection PyBroadException
@@ -77,6 +94,8 @@ def runner_local_main_(args: List[str] = None, token: str = None):
 
     group.add_argument("-C", dest="change", default=None)
 
+    evaluator_name = "local"
+
     parsed = parser.parse_args(args=args)
     quota_cpu = parsed.quota_cpu
     quiet = parsed.quiet
@@ -99,18 +118,18 @@ def runner_local_main_(args: List[str] = None, token: str = None):
         token=token, impersonate=parsed.impersonate, submission_protocols=sub_info.protocols, quiet=quiet,
     )
     if not compat.compatible:
-        msg = "There are no compatible challenges with protocols %s." % sub_info.protocols
-        raise UserError(msg)
+        msg = "There are no compatible challenges with submission protocols."
+        raise UserError(msg, protocols=sub_info.protocols)
 
     dockerfile = os.path.join(path, "Dockerfile")
     if not os.path.exists(dockerfile):
         msg = f'I expected to find the file "{dockerfile}".'
         raise Exception(msg)
 
-    client = check_docker_environment()
+    # client = check_docker_environment()
 
     no_cache = parsed.no_cache
-    no_build = parsed.no_build
+    # no_build = parsed.no_build
     do_pull = not parsed.no_pull
 
     if parsed.challenge is not None:
@@ -125,7 +144,7 @@ def runner_local_main_(args: List[str] = None, token: str = None):
                 raise UserError(msg)
 
             if c not in compat.compatible:
-                msg = 'The challenge "%s" is not compatible with the protocol %s.' % (c, sub_info.protocols,)
+                msg = f'The challenge "{c}" is not compatible with the protocol {sub_info.protocols}.'
                 raise UserError(msg)
 
     if not sub_info.challenge_names:
@@ -156,22 +175,29 @@ For example, you could try:
 
     logger.debug(f"I will evaluate challenge {one}")
     cd = get_challenge_description(token, one, impersonate=parsed.impersonate)
-    # logger.info(cd=cd)
+    # logger.debug(cd=cd)
 
-    tag = "dummy-org/dummy-repo"
-    image = build_image(
-        client, tag=tag, path=path, dockerfile=dockerfile, no_cache=no_cache, no_build=no_build,
-    )
+    # tag = cast(DockerCompleteImageName, "dummy-org/dummy-repo")
+    # TODO: change the build procedure
+    username = "dummy-org"
+    registry = "docker.io"
+
+    solution_container = submission_build_2(username, registry, no_cache)
+
+    # image = build_image(
+    #     client, tag=tag, path=path, dockerfile=dockerfile, no_cache=no_cache, no_build=no_build,
+    # )
 
     caching = False
-    solution_container = tag
+    # solution_container = tag
     SUCCESS = "success"
     steps_ordered = list(sorted(cd.steps))
     logger.debug("steps", steps_ordered=steps_ordered)
     output = parsed.output
     if output is None:
         timestamp = datetime.now().strftime("%y_%m_%d_%H_%M_%S")
-        output = os.path.join("/tmp", one, timestamp)
+
+        output = os.path.join("/tmp/duckietown/dt-challenges-runner/local-evals", one, timestamp)
     logger.info(f"Using output directory {output}")
     for i, challenge_step_name in enumerate(steps_ordered):
         logger.debug(f'Now considering step "{challenge_step_name}"')
@@ -184,7 +210,9 @@ For example, you could try:
         if caching:
             params = os.path.join(wd_final, "parameters.json")
             if os.path.exists(wd_final) and os.path.exists(params):
-                if open(params).read() == evaluation_parameters_str:
+                previous = read_ustring_from_utf8_file(params)
+
+                if previous == evaluation_parameters_str:
                     cr_yaml = open(os.path.join(wd_final, "results.yaml"))
                     cr = ChallengeResults.from_yaml(yaml.load(cr_yaml))
                     if cr.status == SUCCESS:
@@ -194,7 +222,10 @@ For example, you could try:
                         logger.info(msg)
                         continue
                     else:
-                        msg = f'Breaking because step "{challenge_step_name}" was already evaluated with result "{cr.status}".'
+                        msg = (
+                            f'Breaking because step "{challenge_step_name}" was already evaluated with '
+                            f'result "{cr.status}".'
+                        )
                         msg += "\n" + f"If you want to re-do it, erase the directory {wd_final}."
                         logger.error(msg)
                         break
@@ -215,20 +246,16 @@ For example, you could try:
         if os.path.exists(wd):
             shutil.rmtree(wd)
 
-        if not os.path.exists(fd):
-            os.makedirs(fd)
+        mkdirs_thread_safe(fd)
         logger.info(f"Using working dir {wd}")
         params_tmp = os.path.join(wd, "parameters.json")
-        if not os.path.exists(wd):
-            os.makedirs(wd)
-        with open(params_tmp, "w") as f:
-            f.write(evaluation_parameters_str)
+        mkdirs_thread_safe(wd)
+        write_ustring_to_utf8_file(evaluation_parameters_str, params_tmp)
 
         previous = steps_ordered[:i]
         for previous_step in previous:
             pd = os.path.join(wd, CHALLENGE_PREVIOUS_STEPS_DIR)
-            if not os.path.exists(pd):
-                os.makedirs(pd)
+            mkdirs_thread_safe(pd)
 
             d = os.path.join(pd, previous_step)
             # os.symlink('../../%s' % previous_step, d)
@@ -255,6 +282,9 @@ For example, you could try:
             require_scores = set(_.name for _ in cd.scoring.scores)
         else:
             require_scores = set()
+        submission_id = cast(SubmissionID, 0)
+        submitter_name = "local-eval"
+        steps2scores = {}
         cr = run_single(
             wd=wd,
             fd=fd,
@@ -272,6 +302,10 @@ For example, you could try:
             require_scores=require_scores,
             quota_cpu=quota_cpu,
             quiet=quiet,
+            submission_id=submission_id,
+            steps2scores=steps2scores,
+            submitter_name=submitter_name,
+            evaluator_name=evaluator_name,
         )
 
         fn = os.path.join(wd, "results.yaml")
@@ -299,5 +333,46 @@ def dark(x):
     return termcolor.colored(x, attrs=["dark"])
 
 
+def submission_build_2(username: str, registry: str, no_cache: bool = False) -> DockerCompleteImageName:
+    tag = tag_from_date(now_utc())
+    df = "Dockerfile"
+    organization = username.lower()
+    # repository = "aido-submissions"
+    repository = "dummy-repo"
+    update_versions()
+
+    if registry is None:
+        logger.error("had to have explicit registry here")
+        registry = "docker.io"
+
+    complete_image = cast(DockerCompleteImageName, f"{registry}/{organization}/{repository}:{tag}")
+
+    if not os.path.exists(df):
+        msg = f'I expected to find the file "{df}".'
+        raise Exception(msg)
+
+    # cmd = ["docker", "build", "--pull", "-t", complete_image, "-f", df]
+    path = os.getcwd()
+    client = DockerClient.from_env()
+    labels = get_duckietown_labels(path)
+
+    df_contents = read_ustring_from_utf8_file(df)
+
+    build_vars = get_important_env_build_args_dict(df_contents)
+    args = dict(
+        path=path, tag=complete_image, pull=True, buildargs=build_vars, labels=labels, nocache=no_cache
+    )
+    try:
+        run_build(client, **args)
+    except BuildFailed:
+        raise
+
+    # digest = docker_push_optimized(complete_image)
+    # br = parse_complete_tag(digest)
+    # br.tag = tag
+    # return br
+    return complete_image
+
+
 if __name__ == "__main__":
     runner_local_main()
