@@ -15,6 +15,7 @@ class DTShellConstants:
     # DT1_TOKEN_ENV_VARIABLE = 'DUCKIETOWN_TOKEN'
     DT1_TOKEN_CONFIG_KEY = "token_dt1"
     CONFIG_DOCKER_USERNAME = "docker_username"
+    CONFIG_DOCKER_PASSWORD = "docker_password"
     CONFIG_DUCKIETOWN_VERSION = "duckietown_version"
 
 
@@ -22,6 +23,7 @@ class DTShellConstants:
 class ShellConfig:
     token_dt1: Optional[str]  ## key
     docker_username: Optional[str]
+    docker_password: Optional[str]
     duckietown_version: Optional[str]  # daffy, master19, ...
 
 
@@ -44,7 +46,7 @@ def remoteurl_from_RepoInfo(ri: RepoInfo) -> str:
 
 
 def get_shell_config_default() -> ShellConfig:
-    return ShellConfig(token_dt1=None, docker_username=None, duckietown_version=None)
+    return ShellConfig(token_dt1=None, docker_username=None, docker_password=None, duckietown_version=None)
 
 
 def get_config_path() -> str:
@@ -74,6 +76,7 @@ def write_shell_config(shell_config: ShellConfig) -> None:
 DT1_TOKEN_CONFIG_KEY = DTShellConstants.DT1_TOKEN_CONFIG_KEY
 CONFIG_DOCKER_USERNAME = DTShellConstants.CONFIG_DOCKER_USERNAME
 CONFIG_DUCKIETOWN_VERSION = DTShellConstants.CONFIG_DUCKIETOWN_VERSION
+CONFIG_DOCKER_PASSWORD = DTShellConstants.CONFIG_DOCKER_PASSWORD
 
 
 def write_shell_config_to_file(shell_config: ShellConfig, filename: str) -> None:
@@ -112,11 +115,15 @@ def read_shell_config_from_file(fn: str) -> ShellConfig:
     token_dt1 = d.pop(DT1_TOKEN_CONFIG_KEY, None)
     docker_username = d.pop(CONFIG_DOCKER_USERNAME, None)
     duckietown_version = d.pop(CONFIG_DUCKIETOWN_VERSION, None)
+    docker_password = d.pop(CONFIG_DOCKER_PASSWORD, None)
 
     if d:
         msg = f"The config file {fn} contains other options that I do not understand: {d}"
         logger.warning(msg)
 
     return ShellConfig(
-        token_dt1=token_dt1, duckietown_version=duckietown_version, docker_username=docker_username,
+        token_dt1=token_dt1,
+        duckietown_version=duckietown_version,
+        docker_username=docker_username,
+        docker_password=docker_password,
     )
