@@ -3,7 +3,7 @@ import subprocess
 
 from zuper_commons.fs import read_ustring_from_utf8_file
 
-from duckietown_challenges import get_important_env_build_args
+from duckietown_build_utils import get_important_env_build_args
 from . import logger
 
 
